@@ -62,7 +62,7 @@ from pwnlib.elf import constants
 from pwnlib.log import getLogger
 from pwnlib.memleak import MemLeak
 from pwnlib.util.fiddling import enhex
-from pwnlib.util.packing import unpack, _encode
+from pwnlib.util.packing import unpack, _need_bytes
 from pwnlib.util.web import wget
 
 log    = getLogger(__name__)
@@ -75,7 +75,7 @@ def sysv_hash(symbol):
     """
     h = 0
     g = 0
-    for c in bytearray(_encode(symbol)):
+    for c in bytearray(_need_bytes(symbol, 4, 0x80)):
         h = (h << 4) + c
         g = h & 0xf0000000
         h ^= (g >> 24)
@@ -87,7 +87,7 @@ def gnu_hash(s):
 
     Function used to generated GNU-style hashes for strings.
     """
-    s = bytearray(_encode(s))
+    s = bytearray(_need_bytes(s, 4, 0x80))
     h = 5381
     for c in s:
         h = h * 33 + c
@@ -497,7 +497,7 @@ class DynELF(object):
         Returns:
             An ELF object, or None.
         """
-        libc = 'libc.so'
+        libc = b'libc.so'
 
         with self.waitfor('Downloading libc'):
             dynlib = self._dynamic_load_dynelf(libc)
@@ -539,7 +539,7 @@ class DynELF(object):
             lib = 'libc.so'
 
         if symb:
-            symb = _encode(symb)
+            symb = _need_bytes(symb, min_wrong=0x80)
 
         #
         # Get a pretty name for the symbol to show the user
@@ -640,7 +640,7 @@ class DynELF(object):
         while leak.field(cur, LinkMap.l_prev):
             cur = leak.field(cur, LinkMap.l_prev)
 
-        libname = _encode(libname)
+        libname = _need_bytes(libname, 2, 0x80)
 
         while cur:
             self.status("link_map entry %#x" % cur)
@@ -972,12 +972,12 @@ class DynELF(object):
             memsz += (page_size - (memsz % page_size)) % page_size
             pages[vaddr] = leak.n(vaddr, memsz)
 
-        if libs :
-            for lib_name in self.bases() :
-                if len(lib_name) == 0 :
+        if libs:
+            for lib_name in self.bases():
+                if len(lib_name) == 0:
                     continue
                 dyn_lib = self._dynamic_load_dynelf(lib_name)
-                if dyn_lib is not None :
+                if dyn_lib is not None:
                     pages.update(dyn_lib.dump(readonly = readonly))
 
         return pages
